# =================================================================
# =====       XMLShape File Reader                            =====
# =================================================================
#
# This class was created to isolate all about the XMLShape file
# format, so different sintaxis could be used. For instance, the
# original CGAShape sintax can be read, if you provide the parser, 
# of course! :-)

from xml.dom import minidom, Node
from errorMsg import errorMsg

verboseLevel_XMLShapeFileReader = 0   #0 = no verbose, 1 = debug info, 2 = verbose

class XMLShapeFileReader:
	verboseLevel_XMLShapeFileReader = 0

	def __init__(self, fileToRead):
		openedFile = open(fileToRead)
		s = self.clean(openedFile)
		self.XMLShapeFile = minidom.parseString(s)
		self.allRules = []
		self.currentRule = ''
		self.currentPos = 0
		self.currentSuc = ''
		self.currentActionPos = 0
		self.currentAction = ''


	def clean(self, fileToClean):
		foo = fileToClean.read()
		without_n = foo.replace('\n', '')
		without_t = without_n.replace('\t', '')
  
		return without_t

	def getHeader(self):
		node = self.XMLShapeFile.getElementsByTagName('header')[0]
		type = (node.attributes.getNamedItem('type').nodeValue).encode('latin')
		if verboseLevel_XMLShapeFileReader > 0:
			print 'XMLShapeFileReader.header: ', type
		return type
		
	# *****************************************
	#           Rules Iterator
	# *****************************************
	def getNumRules(self):
		count = 0;
		#for each <rule>
		self.allRules = self.XMLShapeFile.getElementsByTagName('rule')
		for rule in self.allRules:
			# Check for empty rules generated: tkngs patch!
			if rule.hasChildNodes():
				count += 1
		return count
				
	def initRules(self):
		if verboseLevel_XMLShapeFileReader > 1:
			print 'XMLShapeFileReader.initRules()'
		self.currentPos = 0
		self.allRules = self.XMLShapeFile.getElementsByTagName('rule')
		self.currentRule = self.allRules[0]
		# Check for empty rules generated: tkngs patch!
		while (not self.currentRule.hasChildNodes()) and not self.finishedRules():
			self.currentPos += 1
			self.currentRule = self.allRules[currentPos]

	def selectNextRule(self):
		self.currentPos += 1
		if not self.finishedRules():
			self.currentRule = self.allRules[self.currentPos]
		# Check for empty rules generated: tkngs patch!
		while (not self.currentRule.hasChildNodes()) and not self.finishedRules():
			self.currentPos += 1
			self.currentRule = self.allRules[self.currentPos]
		if verboseLevel_XMLShapeFileReader > 1:
			print 'XMLShapeFileReader.selectNextRule() - new pos:', self.currentPos

	def finishedRules(self):
		if verboseLevel_XMLShapeFileReader > 1:
			print 'XMLShapeFileReader.finishedRules()'
		return self.currentPos >= self.allRules.length

	def currentRuleId(self):
		return (self.currentRule.attributes.getNamedItem('id').nodeValue).encode('latin')

	def getPredecessor(self):
		return self.getName(self.currentRule.getElementsByTagName('predecessor')[0])

	def getName(self, node):
		return (node.attributes.getNamedItem('name').nodeValue).encode('latin')

	def getValue(self, node):
		return (node.attributes.getNamedItem('value').nodeValue).encode('latin')

	# *****************************************
	#           Action Iterator
	# *****************************************
	def getNumActions(self):
		count = 0;
		#for each <rule>
		self.currentSuc = self.currentRule.getElementsByTagName('successor')[0]
		for action in self.currentSuc.getElementsByTagName('action'):
			count += 1
		return count

	def initActions(self):
		self.currentSuc = self.currentRule.getElementsByTagName('successor')[0]
		self.currentActionPos = 0
		self.currentAction = self.currentSuc.getElementsByTagName('action')[0]

	def finishedActions(self):
		if verboseLevel_XMLShapeFileReader > 1:
			print 'XMLShapeFileReader.finishedActions()'
		return self.currentActionPos >= self.currentSuc.getElementsByTagName('action').length

	def selectNextAction(self):
		self.currentActionPos += 1
		if not self.finishedActions():
			self.currentAction = self.currentSuc.getElementsByTagName('action')[self.currentActionPos]
		if verboseLevel_XMLShapeFileReader > 1:
			print 'XMLShapeFileReader.selectNextAction() - new pos:', self.currentActionPos

	def currentActionName(self):
		return (self.currentAction.attributes.getNamedItem('name').nodeValue).encode('latin')

	# *****************************************
	#           Parameters and Products
	# *****************************************
	def getParameterNames(self):
		nodesList = self.currentAction.getElementsByTagName('param')
		result = []
		for node in nodesList:
			result.append(self.getName(node))
		return result

	def getParameters(self):
		nodesList = self.currentAction.getElementsByTagName('param')
		result = []
		for node in nodesList:
			result.append( (self.getName(node), self.getValue(node)) )
		return result

	def getProducts(self):
		nodesList = self.currentAction.getElementsByTagName('product')
		result = []
		for node in nodesList:
			result.append( self.getName(node) )
		return result

	#******************************************************************************
	#                     hasParameterDefined
	#******************************************************************************
	def hasParameterDefined(self, pName):
		if verboseLevel_XMLShapeFileReader > 1:
			print 'hasParameterDefined: Looking for parameter ', pName, ' in Node parameters'
		params = self.currentAction.getElementsByTagName('param')
		found = False
		for tryName in params:
			if self.getName(tryName) == pName:
				if verboseLevel_XMLShapeFileReader > 1:
					print 'Compulsory: Param found: ', pName
				found = True
		if verboseLevel_XMLShapeFileReader > 1:
			print 'hasParameterDefined: FOUND Values ->', found
		return found


	#******************************************************************************
	#                     readCompulsoryParameters
	#******************************************************************************
	def readCompulsoryParameters(self, parameterNames):
		params = self.currentAction.getElementsByTagName('param')
		results = []
		for parName in parameterNames:
			if verboseLevel_XMLShapeFileReader > 1:
				print 'Compulsory: Looking for parameter ', parName, ' in Node parameters'
			found = False
			for tryName in params:
				if self.getName(tryName) == parName:
					results.append(self.getValue(tryName))
					if verboseLevel_XMLShapeFileReader > 1:
						print 'Compulsory: Param found: ', parName, ' value: ', getValue(tryName)
					found = True
			if (not found):
				text = 'Compulsory: PARAMETER NOT FOUND: ' + parName + ' !!!'
				errorMsg(text)
		if verboseLevel_XMLShapeFileReader > 1:
			print 'Compulsory: FOUND Values ->', results
		return results

	#******************************************************************************
	#                     readOptionalParameters
	#******************************************************************************
	def readOptionalParameters(self, parameterList):
		params = self.currentAction.getElementsByTagName('param')
		results = []
		for (pname, pvalue) in parameterList:
			if verboseLevel_XMLShapeFileReader > 1:
				print 'Optional: Looking for parameter ', pname, ' in Node parameters'
			found = False
			for tryName in params:
				if self.getName(tryName) == pname:
					results.append(self.getValue(tryName))
					if verboseLevel_XMLShapeFileReader > 1:
						print 'Optional: Param found: ', pname, ' value: ', getValue(tryName)
					found = True
			if (not found):
				results.append(pvalue)
				if verboseLevel_XMLShapeFileReader > 1:
					print 'Optional: Param  not found: ', pname, ' value', pvalue
		if verboseLevel_XMLShapeFileReader > 1:
			print 'Optional: Final Values ->', results
		return results

		

#******************************************************************************
#                     test_XMLShapeFileReader
#******************************************************************************
def print_XMLShapeFile(fileToRead):
	fileReader = XMLShapeFileReader(fileToRead)
	if verboseLevel_XMLShapeFileReader > 0:
		print 'XMLShapeFileReader.getNumRules() -> ', fileReader.getNumRules(), ' rules read'

	fileReader.initRules()
	while not fileReader.finishedRules():
		print 'Processing Rule (id=',fileReader.currentRuleId(), ') with ', fileReader.getNumActions(), ' actions'
		print '   Predecessor: ',fileReader.getPredecessor()
		fileReader.initActions()
		while not fileReader.finishedActions():
			print '   Action: ', fileReader.currentActionName()
			fileReader.selectNextAction()
			print fileReader.currentActionName(), '(',
			parameters = fileReader.getParameters()
			for (pname, pvalue) in parameters:
				print pname, '=', pvalue, ',',
			print ') {',
			products = fileReader.getProducts()
			for prod in products:
				print prod, ','
			print '}'
		fileReader.selectNextRule()



#******************************************************************************
#                     XMLShape2CGAShape
#******************************************************************************
def XMLShape2CGAShape(fileToRead):
	fileReader = XMLShapeFileReader(fileToRead)
	print 'File Type: --', fileReader.getHeader(), '--'
	print
	if verboseLevel_XMLShapeFileReader > 0:
		print 'XMLShapeFileReader.getNumRules() -> ', fileReader.getNumRules(), ' rules read'

	fileReader.initRules()
	while not fileReader.finishedRules():
		print fileReader.currentRuleId(), ': ', fileReader.getPredecessor(), ' ->',
		numActions = fileReader.getNumActions()
		if  numActions > 1:
			print
		fileReader.initActions()
		while not fileReader.finishedActions():
			if numActions > 1:
				print '        ',
			print fileReader.currentActionName(), '(',
			parameters = fileReader.getParameters()
			for (pname, pvalue) in parameters:
				print pvalue, ',',
			print ') {',
			products = fileReader.getProducts()
			for prod in products:
				print prod, ',',
			print '}'
			fileReader.selectNextAction()
		fileReader.selectNextRule()


def buildXMLShapeFileReader(fileToRead):
	print 'Creating buildXMLShapeFileReader'
	fileReader = XMLShapeFileReader(fileToRead)
	return fileReader